% analyseAngularResponse
% Check the behaviour or marker with respect to angulation.
% Assume that the marker points are defined in a plane z = cst.
% points: 3 x n array of double
%         each column represents the x,y,z coordinates of a point in mm
% z: z distance in mm where the analysis should take place.
% handle: contains the resulting figure

function handle = analyseMarkerAngularResponse (points, z, noiseAmplitute, showPlots)

    % These functions are not defined by default in Octave
    function rad = deg2rad(deg)
        rad = deg*pi/180;
    end

    function deg = rad2deg (x)
        deg = mod ((x / (2 * pi)) * 360, 360);
    end
    
    if nargin < 3,
        error('Need three input arguments');
    end;

    if exist ('showPlots','var') ~= 1
        showPlots = 1;
    end

    if showPlots == 1
        handle = figure;
    else
        handle = figure('visible', 'off');
    end

    clf;

    trans = [0;0;z];
    s = size(points,2);
    markerMaxAngulationDeg = 60;

    % Analyse when rotating around Z axis

    anglesRotZDeg   = 0:359;
    errorRegRotZmm  = zeros(1, length (anglesRotZDeg));
    errorAngRotZDeg = zeros(1, length (anglesRotZDeg));
    for i=1:length(anglesRotZDeg)
        Rot = rodrigues([0 0 1] * deg2rad(anglesRotZDeg(i)));

        measuredPoints = Rot * points + repmat (trans,1,s);
        [RegRot, RegTrans, RegError] = markerRegistration(points, measuredPoints);

        measuredPointsWithNoise = markerAddNoise (measuredPoints, noiseAmplitute);
        [RegRotWithNoise, RegTransWithNoise, RegErrorWithNoise] = markerRegistration(points, measuredPointsWithNoise);

        errorRegRotZmm(i)  = RegErrorWithNoise;
        errorAngRotZDeg(i) = rad2deg (norm(rodrigues(RegRot*RegRotWithNoise')));

    end


    % Analyse when rotating around Y axis

    anglesRotYDeg   = -markerMaxAngulationDeg:markerMaxAngulationDeg;
    errorRegRotYmm  = zeros(1, length (anglesRotYDeg));
    errorAngRotYDeg = zeros(1, length (anglesRotYDeg));
    for i=1:length(anglesRotYDeg)
        Rot = rodrigues([0 1 0] * deg2rad(anglesRotYDeg(i)));

        measuredPoints = Rot * points + repmat (trans,1,s);
        [RegRot, RegTrans, RegError] = markerRegistration(points, measuredPoints);

        measuredPointsWithNoise = markerAddNoise (measuredPoints, noiseAmplitute);
        [RegRotWithNoise, RegTransWithNoise, RegErrorWithNoise] = markerRegistration(points, measuredPointsWithNoise);

        errorRegRotYmm(i)  = RegErrorWithNoise;
        errorAngRotYDeg(i) = rad2deg (norm(rodrigues(RegRot*RegRotWithNoise')));

    end

     % Analyse when rotating around X axis

    anglesRotXDeg   = -markerMaxAngulationDeg:markerMaxAngulationDeg;
    errorRegRotXmm  = zeros(1, length (anglesRotXDeg));
    errorAngRotXDeg = zeros(1, length (anglesRotXDeg));
    for i=1:length(anglesRotXDeg)
        Rot = rodrigues([1 0 0] * deg2rad(anglesRotXDeg(i)));

        measuredPoints = Rot * points + repmat (trans,1,s);
        [RegRot, RegTrans, RegError] = markerRegistration(points, measuredPoints);

        measuredPointsWithNoise = markerAddNoise (measuredPoints, noiseAmplitute);
        [RegRotWithNoise, RegTransWithNoise, RegErrorWithNoise] = markerRegistration(points, measuredPointsWithNoise);

        errorRegRotXmm(i)  = RegErrorWithNoise;
        errorAngRotXDeg(i) = rad2deg (norm(rodrigues(RegRot*RegRotWithNoise')));

    end

    report = [];
    results = zeros(3,4);

    subplot (3,2,1);
        hold on;
            plot (anglesRotXDeg, errorRegRotXmm);
            xlabel('Rotation Angle around X axis [deg]');
            ylabel('Expected Registration Error [mm]');
            report{end+1} = sprintf ('Simulation of reg. errors when rotating around the X axis: mean(%f), std(%f)', mean(errorRegRotXmm), std(errorRegRotXmm));
            results(1,1:2) = [mean(errorRegRotXmm), std(errorRegRotXmm)];
            title (report{end});
        hold off;
    subplot (3,2,2);
        hold on;
            plot (anglesRotXDeg, errorAngRotXDeg, 'b');
            xlabel('Rotation Angle around X axis [deg]');
            ylabel('Registration rotationnal error [deg]');
            report{end+1} = sprintf ('Simulation of angular errors when rotating around the X axis: mean(%.4f), std(%.4f)', mean(errorAngRotXDeg), std(errorAngRotXDeg));
            results(1,3:4) = [mean(errorAngRotXDeg), std(errorAngRotXDeg)];
            title (report{end});
        hold off;
    subplot (3,2,3);
        hold on;
            plot (anglesRotYDeg, errorRegRotYmm);
            xlabel('Rotation Angle around Y axis [deg]');
            ylabel('Expected Registration Error [mm]');
            report{end+1} = sprintf ('Simulation of reg. errors when rotating around the Y axis: mean(%.4f), std(%.4f)', mean(errorRegRotYmm), std(errorRegRotYmm));
            results(2,1:2) = [mean(errorRegRotYmm), std(errorRegRotYmm)];
            title (report{end});
        hold off;
    subplot (3,2,4);
        hold on;
            plot (anglesRotYDeg, errorAngRotYDeg, 'b');
            xlabel('Rotation Angle around Y axis [deg]');
            ylabel('Registration rotationnal error [deg]');
            report{end+1} = sprintf ('Simulation of angular errors when rotating around the Y axis: mean(%.4f), std(%.4f)', mean(errorAngRotYDeg), std(errorAngRotYDeg));
            results(2,3:4) = [mean(errorAngRotYDeg), std(errorAngRotYDeg)];
            title (report{end});
        hold off;
    subplot (3,2,5);
        hold on;
            plot (anglesRotZDeg, errorRegRotZmm);
            xlabel('Rotation Angle around Z axis [deg]');
            ylabel('Expected Registration Error [mm]');
            report{end+1} = sprintf ('Simulation of reg. errors when rotating around the Z axis: mean(%.4f), std(%.4f)', mean(errorRegRotZmm), std(errorRegRotZmm));
            results(3,1:2) = [mean(errorRegRotZmm), std(errorRegRotZmm)];
            title (report{end});
        hold off;
    subplot (3,2,6);
        hold on;
            plot (anglesRotZDeg, errorAngRotZDeg, 'b');
            xlabel('Rotation Angle around Z axis [deg]');
            ylabel('Registration rotationnal error [deg]');
            report{end+1} = sprintf ('Simulation of angular errors when rotating around the Z axis: mean(%.4f), std(%.4f)', mean(errorAngRotZDeg), std(errorAngRotZDeg));
            results(3,3:4) = [mean(errorAngRotZDeg), std(errorAngRotZDeg)];
            title (report{end});
        hold off;

   displayMatrix(results, {'X','Y','Z'}, {'RegErrMM (mean)', 'RegErrMM (std)','RotErrDeg (mean)', 'RotErrDeg (std)'});


   %cellfun(@(x)disp(x),report);

end
