function [nbErrorPbs, nbWarningPbs] = generateMarkerCombinationReport(markers, threshold, reportFilename)
% generateMarkerCombinationReport generates a marker combination analysis report
% markers: struct array
%          The fields 'points' and 'name' are expected
%         'points' must be a 3 x n array of the carthesian x,y,z coordinates
% threshold: double
% reportFilename: 1 x n char
%                 The full filesystem path where the generated report shall
%                 be written to. The implementation uses matlab's diary() to store
%                 command window output to the log file.
%                 If the file already exists, the report is appended.
% returns
% nbErrorPbs: the number of errors
% nbWarningPbs: the number of warnings

    silent = exist ('reportFilename', 'var') == 0;

    if ~silent
        diary(reportFilename);
        fid = fopen(reportFilename, 'a+');
        diary on;

        disp('*************** Atracsys marker combination analyzer ***************');
        disp('*** Input ***');
    end
    nbMarkers = size(markers, 2);
    nbSegments = 0;
    nbTriangleDistanceMaps = 0;
    nbTriangleDistanceMapComparisons = 0;
    for i=1:nbMarkers
        segments = computeSegments(markers(i).points);
        markers(i).triangles = computeTriangles(getSegmentLengthMatrix(segments));
        markers(i).triangleDistanceMaps = computeTriangleDistanceMaps(markers(i).triangles, getSegmentLengthMatrix(segments));
        if ~silent
            fprintf('Marker %i: "%s" (%i segments, %i triangles) \n', ...
                i, markers(i).name, size(segments, 2), size(markers(i).triangles, 2));
        end
        for j=1:size(segments, 2)
            nbSegments = nbSegments + 1;
            segments(j).markerIdx = i;
            segmentCollection(nbSegments) = segments(j);
        end

        for j=1:size(markers(i).triangleDistanceMaps, 2)
            nbTriangleDistanceMaps = nbTriangleDistanceMaps + 1;
            triangleDistanceMapList(nbTriangleDistanceMaps) = markers(i).triangleDistanceMaps(j);
            triangleDistanceMapList(nbTriangleDistanceMaps).triangleIdx = sprintf('M%iT%i', i, triangleDistanceMapList(nbTriangleDistanceMaps).triangleIdx);
        end
        nbTriangleDistanceMapComparisons = nbTriangleDistanceMapComparisons - nchoosek(size(markers(i).triangleDistanceMaps, 2), 2);
    end
    nbTriangleDistanceMapComparisons = nbTriangleDistanceMapComparisons + nchoosek(nbTriangleDistanceMaps ,2);

    if ~silent
        fprintf('\n*** Segment length distribution ***\n');
    end
    segmentCollection = orderfields(segmentCollection, {'markerIdx', 'point1Idx', 'point2Idx', 'length'});
    [~, index] = sort([segmentCollection.length]);
    segmentCollection = segmentCollection(index);
    if ~silent
        displayMatrix(segmentCollection, '#%i', {'Marker idx', 'Point 1 idx', 'Point 2 idx', 'Length'});
        fprintf('*** Triangle distance map distribution ***\n');
    end
	  %This kind of sorting only works on MATLAB
    if ( exist('OCTAVE_VERSION', 'builtin') == 0 )
    	[~, index] = sortrows([{triangleDistanceMapList.distance2}', {triangleDistanceMapList.distance1}'], [2 1]);
    end
    triangleDistanceMapList = triangleDistanceMapList(index);
    if ~silent
        displayMatrix(triangleDistanceMapList, '#%i', {'Mkr/Tr idx', 'Ref. pt. idx', 'Distance 1', 'Distance 2', 'Opp. seg. len.'});
    end
    %disp(sprintf('*** Triangle distance maps ***'));
    %nbCmps = nchoosek(nbTriangleDistanceMaps, 2);       % Includes intra-marker comparisons, they are substracted later
    %for i=1:nbMarkers
        %disp(sprintf('Marker %i: "%s"', i, markers(i).name));
        %disp(sprintf('Triangles: %i - Triangle distance maps: %i', size(markers(i).triangles, 2), size(markers(i).triangles, 2) * 3));
        %dispTriangles(markers(i).triangles);
        %disp([{'Triangle idx', 'Ref. pt. idx', 'Distance 1', 'Distance 2', 'Opp. seg. length'}; permute(squeeze(struct2cell(markers(i).triangleDistanceMaps)), [2 1])]);
        %nbCmps = nbCmps - nchoosek(size(markers(i).triangleDistanceMaps, 2), 2);
    %end

    if ~silent
        disp('##### Marker combination analysis #####');
        fprintf('Triangle distance maps: %i - Triangle distance map combinations (inter-marker): %i\n', nbTriangleDistanceMaps, nbTriangleDistanceMapComparisons);
    end
    nbErrorPbs   = 0;
    nbWarningPbs = 0;
    for i=1:nbMarkers
        for j=1:size(markers(i).triangleDistanceMaps, 2)
            for k=i+1:nbMarkers
                for l=1:size(markers(k).triangleDistanceMaps, 2)
                    %disp(sprintf('marker %i distance map %i vs. marker %i distance map %i', i, j, k, l));
                    if compareTriangleDistanceMaps(markers(i).triangleDistanceMaps(j), markers(k).triangleDistanceMaps(l), threshold)
                        triangle1 = markers(i).triangles(markers(i).triangleDistanceMaps(j).triangleIdx);
                        triangle2 = markers(k).triangles(markers(k).triangleDistanceMaps(l).triangleIdx);
                        if abs(markers(i).triangleDistanceMaps(j).oppositeSegmentLength - markers(k).triangleDistanceMaps(l).oppositeSegmentLength) < threshold / 2
                            nbErrorPbs = nbErrorPbs + 1;
                            if ~silent
                                diary off
                                warning('Problem %i: marker %i triangle %i (%i, %i, %i) and marker %i triangle %i (%i, %i, %i) are similar\n', nbErrorPbs, i, markers(i).triangleDistanceMaps(j).triangleIdx, triangle1.point1Idx, triangle1.point2Idx, triangle1.point3Idx, k, markers(k).triangleDistanceMaps(l).triangleIdx, triangle2.point1Idx, triangle2.point2Idx, triangle2.point3Idx);
                                diary on
                                fprintf(fid, 'Problem %i: marker %i triangle %i (%i, %i, %i) and marker %i triangle %i (%i, %i, %i) are similar\n', nbErrorPbs, i, markers(i).triangleDistanceMaps(j).triangleIdx, triangle1.point1Idx, triangle1.point2Idx, triangle1.point3Idx, k, markers(k).triangleDistanceMaps(l).triangleIdx, triangle2.point1Idx, triangle2.point2Idx, triangle2.point3Idx);
                            end
                        else
                            if ~silent
                                fprintf('Note: marker %i triangle %i (%i, %i, %i) and marker %i triangle %i (%i, %i, %i) have two similar segments\n', i, markers(i).triangleDistanceMaps(j).triangleIdx, triangle1.point1Idx, triangle1.point2Idx, triangle1.point3Idx, k, markers(k).triangleDistanceMaps(l).triangleIdx, triangle2.point1Idx, triangle2.point2Idx, triangle2.point3Idx);
                            end
                        end
                        nbWarningPbs = nbWarningPbs + 1;
                        if ~silent
                            fprintf('\tSegment lengths: M%iT%i (%f, %f, %f), M%iT%i (%f, %f, %f)\n', i, markers(i).triangleDistanceMaps(j).triangleIdx, markers(i).triangleDistanceMaps(j).distance1, markers(i).triangleDistanceMaps(j).distance2, markers(i).triangleDistanceMaps(j).oppositeSegmentLength, k, markers(k).triangleDistanceMaps(l).triangleIdx, markers(k).triangleDistanceMaps(l).distance1, markers(k).triangleDistanceMaps(l).distance2, markers(k).triangleDistanceMaps(l).oppositeSegmentLength);
                        end
                    end
                end
            end
        end
    end

    if (nbErrorPbs == 0) && ~silent
        disp('No marker combination problems detected');
    end

    if ~silent
        diary off
        fclose(fid);
    end
end
