function [nbWarningPbs] = generateMarkerReport(points, threshold, reportFilename, markerName, showPlots)
% generateMarkerReport generates a marker analysis report
% points: 3 x n array of double
%         each column represents the x,y,z coordinates of a point
% threshold: double
% reportFilename: 1 x n char
%                 The full filesystem path where the generated report shall
%                 be written to. The implementation uses matlab's diary() to store
%                 command window output to the log file.
%                 If the file already exists, the report is appended.
% markerName: 1 x n char
%             If it's one of the standard markers defined in getMarkerName(), this
%             field should be left empty
% showPlots: boolean
% returns: nbWarningPbs the number of warnings

    if ~exist ('reportFilename', 'var') || ...
       ~exist ('markerName', 'var')
        silent = true;
    else
        silent = false;
    end

    if ~exist ('showPlots', 'var')
        showPlots = false;
    end

    function header1 (txt)
        if ~silent
            fprintf('*************** %s ***************\n\n', txt);
        end
    end

    function header2 (txt)
        if ~silent
            fprintf('*** %s ***\n\n', txt);
        end
    end

    function paragraph (txt)
        if ~silent
            fprintf('%s\n\n', txt);
        end
    end

    % *************   Computations   **************
    nbPoints = size(points,2);
    segments = computeSegments(points);
    nbSegments = size(segments, 2);

    segmentLengths = getSegmentLengthsAsArray(segments);
    segmentLengthMatrix = getSegmentLengthMatrix(segments);
    segmentLengthDifferences = computeSegmentLengthDifferences(segments);

    triangles = computeTriangles(segmentLengthMatrix);
    nbTriangles = nchoosek(nbPoints, 3);
    triangleDistanceMaps = computeTriangleDistanceMaps(triangles, segmentLengthMatrix);

    % *************  Reporting  ***************
    if ~silent

        diary(reportFilename);
        fid = fopen(reportFilename, 'a+');
        diary on;
        header1 (sprintf ('Marker ''%s'' analysis',markerName));

        header2 ('Parameters');
        paragraph(sprintf('  Distance Threshold: %.2f', threshold));

        header2 ('Fiducial Coordinates (from input file)');
        displayMatrix(points, {'x' 'y' 'z'}, 'Point %i');
        paragraph (sprintf('  Segments: %i - Triangles: %i', nbSegments, nbTriangles));

        header2 ('Segments Analysis');
        paragraph('  Segment length matrix:');
        displayMatrix(segmentLengthMatrix, 'Point %i', 'Point %i');

        paragraph('  Segment lengths sorted in ascending order:');

    end

    sortedLengths = sort(segmentLengths);
    segmentDistances = [sortedLengths, 0] - [0, sortedLengths];
    segmentDistances(end) = [];

    if ~silent
        disp(sortedLengths);

        paragraph(sprintf ('  Segment inter-distances (minimum: %.2f):', min(segmentDistances)));
        disp(segmentDistances);

        if min(segmentDistances) < threshold
            warning ('Distance threshold defined at %.2f, and minimim distance between segments is %.2f.', ...
                threshold, min(segmentDistances));
        end
    end



    %disp(sprintf('Segment length distribution ranges computed using the +/- half the threshold (%i mm) - overlapping ranges are merged', threshold));
    %ranges = computeRanges(sortedLengths, threshold);
    %disp('"Blocked" ranges:');
    %dispRanges(ranges);

    if exist('gaussmf') == 2
        h = plotSegmentLengthDistribution(segmentLengths, threshold, showPlots);
        [pathstr,filename] = fileparts(reportFilename);
        saveas(h, sprintf('%s/%s lenght dist.jpg', pathstr, markerName));
        paragraph(sprintf('The segment length distribution plot was saved to %s/%s.jpg\n', pathstr, filename));
    else
        paragraph('The segment length distribution plot was not be generated because the Matlab gaussmf function is not available.');
    end

    header2('Segment distribution analysis');
    paragraph(sprintf('  Segment combinations: %i\n', nchoosek(nbSegments, 2)));

    nbWarningPbs = 0;
    nbPbsPerPt = zeros([1 nbPoints]);
    for i=1:size(segmentLengthDifferences, 2)
        if(abs(segmentLengthDifferences(i).difference) < threshold)
            nbWarningPbs = nbWarningPbs + 1;
            if ~silent
                %fprintf('Problem %i: seg(%i, %i) vs. seg(%i, %i): %f - %f = %f\n', nbPbs, segments(segmentLengthDifferences(i).segment1Idx).point1Idx, segments(segmentLengthDifferences(i).segment1Idx).point2Idx, segments(segmentLengthDifferences(i).segment2Idx).point1Idx, segments(segmentLengthDifferences(i).segment2Idx).point2Idx, segments(segmentLengthDifferences(i).segment1Idx).length, segments(segmentLengthDifferences(i).segment2Idx).length, segmentLengthDifferences(i).difference)
                diary off
                warning('Problem %i - segment length difference below threshold: seg(%i, %i) vs. seg(%i, %i): %f - %f = %f \n', nbWarningPbs, segments(segmentLengthDifferences(i).segment1Idx).point1Idx, segments(segmentLengthDifferences(i).segment1Idx).point2Idx, segments(segmentLengthDifferences(i).segment2Idx).point1Idx, segments(segmentLengthDifferences(i).segment2Idx).point2Idx, segments(segmentLengthDifferences(i).segment1Idx).length, segments(segmentLengthDifferences(i).segment2Idx).length, segmentLengthDifferences(i).difference);
                diary on
                fprintf(fid, 'Problem %i - segment length difference below threshold: seg(%i, %i) vs. seg(%i, %i): %f - %f = %f \n', nbWarningPbs, segments(segmentLengthDifferences(i).segment1Idx).point1Idx, segments(segmentLengthDifferences(i).segment1Idx).point2Idx, segments(segmentLengthDifferences(i).segment2Idx).point1Idx, segments(segmentLengthDifferences(i).segment2Idx).point2Idx, segments(segmentLengthDifferences(i).segment1Idx).length, segments(segmentLengthDifferences(i).segment2Idx).length, segmentLengthDifferences(i).difference);
                nbPbsPerPt(segments(segmentLengthDifferences(i).segment1Idx).point1Idx) = nbPbsPerPt(segments(segmentLengthDifferences(i).segment1Idx).point1Idx) + 1;
                nbPbsPerPt(segments(segmentLengthDifferences(i).segment1Idx).point2Idx) = nbPbsPerPt(segments(segmentLengthDifferences(i).segment1Idx).point2Idx) + 1;
                nbPbsPerPt(segments(segmentLengthDifferences(i).segment2Idx).point1Idx) = nbPbsPerPt(segments(segmentLengthDifferences(i).segment2Idx).point1Idx) + 1;
                nbPbsPerPt(segments(segmentLengthDifferences(i).segment2Idx).point2Idx) = nbPbsPerPt(segments(segmentLengthDifferences(i).segment2Idx).point2Idx) + 1;
            end
        end
    end
    if nbWarningPbs == 0
        paragraph('No segment length distribution / symetry problems detected');
    else
        if ~silent
            paragraph('Amount of times each point appears in a problematic constellation:');
            displayMatrix(nbPbsPerPt, {'Problem count'}, 'Pt. %i');
        end
    end

    if ~silent
        header2('Triangles');
        displayTriangles(triangles);

        header2('Triangle distance maps');
        displayMatrix(triangleDistanceMaps, '#%i', {'Triangle idx', 'Ref. pt. idx', 'Distance 1', 'Distance 2', 'Opp. seg. length'});

        header2('Design help');
        paragraph('  "Open" segment length ranges:');
        displayMatrix(invertRanges(computeRanges(sortedLengths, threshold)), 'Range %i', {'Lower', 'Upper'});

        header2 ('Static angular error for fTk500 at position [0,0,1500] mm (expressed as a Rodrigues vector)')
        analyseMarkerAngularErrorWhenStatic (points,1500,[0.0230; 0.0324; 0.1641]);

        diary off
        fclose(fid);
    end

    %header2('Angular error simulation');
    % Compute marker for cTk @ x=y=0, z=1000mm
    %analyseMarkerAngularResponse (points,1000,[0.0064 0.0148 0.0449], 0);
    % h = analyseMarkerAngularResponse (points,1000,[0.0064 0.0148 0.0449], 0);
    % Compute marker for fTk 500 @ x=y=0, z=1500mm
    % h = analyseMarkerAngularResponse (points,1500,[0.0230; 0.0324; 0.1641], 0);
    % Compute marker for fTk 250 @ x=y=0, z=1500mm
    %h = analyseMarkerAngularResponse (points,1500,[0.0140;0.0156;0.3126]);
    if showPlots
        h.Visible = 'on'; % Following resize does not work if figure is hidden
            set(h,'PaperUnits','normalized','PaperPosition',[0 0 1.67 1]);
            set(h,'units','normalized','outerposition',[0 0 1 1]);
            [pathstr] = fileparts(reportFilename);
            print(sprintf('%s/%s angular analysis.jpg', pathstr, markerName), '-dpng', '-r0');
        h.Visible = 'off';
    end

end
